""" Testing math_dollar module
"""

from ..math_dollar import d2m_source, rst_dollars_to_math


def source_shim(in_str):
    # Shim to return source, so testing a bit more convenient
    source = [in_str]
    d2m_source(source)
    return "\n".join(source)


def test_dollars_to_math():
    # Simple inline math
    d2m = rst_dollars_to_math
    # For a few simple inputs, test list and string versions
    for f in (source_shim, d2m):
        assert f('Here be $a = b$ math') == 'Here be :math:`a = b` math'
        assert f('Here be $a =\nb$ math') == 'Here be :math:`a =\nb` math'
    # Dollars inside backticks don't get replaced
    assert d2m('Here be `$a = b$` math') == 'Here be `$a = b$` math'
    assert d2m('Here be ``$a = b$`` math') == 'Here be ``$a = b$`` math'
    assert d2m('Here `$a = b$` and `$c$`') == 'Here `$a = b$` and `$c$`'
    # In general this is because dollars before or after backticks stay same
    assert d2m('Here be `$a = b` math') == 'Here be `$a = b` math'
    assert d2m('Here be `a = b$` math') == 'Here be `a = b$` math'
    # And in general that's because everything between backticks stays the same
    assert d2m('Such as ``$1 $2 $3`` etc') == 'Such as ``$1 $2 $3`` etc'
    assert d2m('Such as `$1 $2 $3` etc') == 'Such as `$1 $2 $3` etc'
    # Can mix with and without backticks
    assert d2m('Here `$a$` and $c$') == 'Here `$a$` and :math:`c`'
    # Dollars inside curlies don't get replaced
    assert (d2m(r'Now $f(n) = 0 \text{ if $n$ is prime}$ then') ==
                 r'Now :math:`f(n) = 0 \text{ if $n$ is prime}` then')
    # We do now mathize dollars on lines with indents
    assert d2m(' $env$') == ' :math:`env`'
    assert d2m('::\n  $env\n  $var') == '::\n  :math:`env\n  `var'
    assert (d2m('::\n  $env$\n\nHere $b$ there') ==
                '::\n  :math:`env`\n\nHere :math:`b` there')
    assert (d2m('and some `$real dollars`.  More ``$real dollars``.') ==
            'and some `$real dollars`.  More ``$real dollars``.')
    assert (d2m('\n\nSome $math dollars$.  More $math dollars$.\n') ==
            '\n\nSome :math:`math dollars`.  More :math:`math dollars`.\n')
    # Lots of backticks in a heading
    assert (d2m(
        '\nWord\n\nHeading\n```````\nMore ``stuff`` here') ==
        '\nWord\n\nHeading\n```````\nMore ``stuff`` here')
    mysterious_problem="""\
Some text short
```````````````

::

    a
    b
    c
    d

``$H``
"""
    assert d2m(mysterious_problem) == mysterious_problem
    # Test case where text with markers may be substituted with another marker.
    # This depends on dictionary ordering, so will only sometimes catch errors.
    assert (d2m(
        'She `is in ``her own`` new ``world``, always` leaving') ==
        'She `is in ``her own`` new ``world``, always` leaving')
    assert (d2m(
        "* A list item containing\n  $f = 6$ some mathematics.") ==
        "* A list item containing\n  :math:`f = 6` some mathematics.")
    assert (d2m(
        "* A list item containing ``a literal across\n  lines`` and also "
        "$g = 7$ some mathematics.") ==
        "* A list item containing ``a literal across\n  lines`` and also "
        ":math:`g = 7` some mathematics.")



""" Tests for plotdirective build using sphinx extensions

Test ability to combine plot_directive with mathcode
"""

from os.path import dirname, join as pjoin
import re

from six import PY3

import sphinx

SPHINX_ge_1p8 = sphinx.version_info[:2] >= (1, 8)
SPHINX_ge_1p7 = sphinx.version_info[:2] >= (1, 7)
SPHINX_ge_1p5 = sphinx.version_info[:2] >= (1, 5)

from sphinxtesters import PageBuilder

PAGES = pjoin(dirname(__file__), 'plotdirective')


def format_math_block(name, latex, label=None, number=None,
                      ids=None):
    """ Simulate math block output from Sphinx at different versions
    """
    if SPHINX_ge_1p8:
        number = 'True' if number is None else number
        label = 'True' if label is None else label
        id_part = '' if ids is None else 'ids="{}" '.format(ids)
        return (
            '<math_block docname="{}" {}label="{}" '
            'nowrap="False" number="{}" xml:space="preserve">{}'
            '</math_block>'.format(name, id_part, label, number, latex)
        )
    number = 'None' if number is None else number
    label = 'None' if label is None else label
    # Sphinx >= 1.5 has number="" clause in parameters
    number_part = ' number="{}"'.format(number) if SPHINX_ge_1p5 else ''
    u_prefix = '' if PY3 or SPHINX_ge_1p7 else 'u'
    id_part = '' if ids is None else """ids="[{}'{}']" """.format(
        u_prefix, ids)
    return (
        '<displaymath docname="{}" {}label="{}" '
        'latex="{}" nowrap="False"{}/>'.format(
            name, id_part, label, latex, number_part)
    )

EXP_PLOT_AND_MATH = (
    '<title>Plot directive with mathcode</title>\n'
    '<paragraph>Some text</paragraph>\n'
    r'<literal_block (highlight_args="{}"\s*)?language="python" '
    'linenos="False" xml:space="preserve">a = 101</literal_block>\n'
    '<only expr="html"/>\n'
    '<only expr="latex"/>\n'
    '<only expr="texinfo"/>\n'
    '<paragraph>More text</paragraph>\n'
    + format_math_block('plot_and_math', '101'))


class TestPlotDirective(PageBuilder):
    # Test build and output of custom_plotdirective project
    page_source_template = PAGES

    def test_plot_and_math(self):
        doctree = self.get_doctree('plot_and_math')
        assert len(doctree.document) == 1
        tree_str = self.doctree2str(doctree)
        assert re.compile(EXP_PLOT_AND_MATH).search(tree_str)


class TestTopPlotDirective(TestPlotDirective):
    # Test we can import mathcode with just `texext`

    @classmethod
    def modify_source(cls):
        conf_fname = pjoin(cls.page_source, 'conf.py')
        with open(conf_fname, 'rt') as fobj:
            contents = fobj.read()
        contents = contents.replace("'texext.mathcode'", "'texext'")
        with open(conf_fname, 'wt') as fobj:
            fobj.write(contents)



""" Test building of docstrings into pages with autodoc
"""

import re

from sphinxtesters import SourcesBuilder

# Utilities for debugging
# py.test texext/tests/test_docstrings.py --pdb
# write_file(html)
# IPython: run texext/tests/test_docstrings.py
# check_re('some_regexp_pattern')

def write_file(contents, fname='.html.txt'):
    with open(fname, 'wt') as fobj:
        fobj.write(contents)


def read_file(fname='.html.txt'):
    with open(fname, 'rt') as fobj:
        contents = fobj.read()
    return contents


def check_re(pattern, text=None):
    text = read_file() if text is None else text
    full_pattern = pattern.format(**FRAGMENTS)
    return re.search(full_pattern, text)


# Docstring build regexp
# sphinx 1.1.3 has <tt> for <code>
# sphinx 1.7b0 has "math notranslate" for "math"
FRAGMENTS = dict(code=r'(code|tt)',
                 cclass=r'"(docutils literal( notranslate)?)"',
                 math=r'"math( notranslate)?( nohighlight)?"')
DOCSTRING_RE = re.compile(
r'<p>Here is the module docstring:</p>\n'
r'<span class="target" id="module-texext.tests.for_docstrings"></span>'
r'<p>A module to test docstring parsing with math such as '
r'<span class={math}'
r'>\\\(\\gamma = \\cos\(\\alpha\)\\\)</span></p>\n'
r'<p>Need to test other markup - so: '
r'<a class="reference external" href="https://github\.com">a link</a>\.</p>\n'
r'<p>Here is the <{code} class={cclass}>'
r'<span class="pre">func</span></{code}> docstring:</p>\n'
r'<dl class="function">\n'
r'<dt id="texext.tests.for_docstrings\.func">\n'
r'<{code} class="descclassname">texext.tests.for_docstrings\.'
r'</{code}>'
r'<{code} class="descname">func</{code}>'
'('  # beginning of regexp group
r'<span class="sig-paren">\(</span><span class="sig-paren">\)</span>'
'|'
r'<big>\(</big><big>\)</big>'  # sphinx 1.1.3
')'  # end of regexp group
r'<a class="headerlink" href="#texext.tests.for_docstrings\.func" '
r'title="Permalink to this definition">.+</a></dt>\n'
r'<dd><p>A docstring with math in first line '
r'<span class={math}>\\\(z = \\beta\\\)</span></p>\n'
r'<p>With some more <span class={math}>\\\(a = 1\\\)</span> math\. '
r'Math across lines - <span class={math}>\\\(b\n'
r'= 2\\\)</span>\.</p>\n'
r'<p>Further, there is a <a class="reference external" '
r'href="https://python\.org">link</a> and a substituted\.</p>\n'
'</dd></dl>'.format(**FRAGMENTS))


class TestDocstrings(SourcesBuilder):

    rst_sources = dict(a_page="""\
Here is the module docstring:

.. automodule:: texext.tests.for_docstrings

Here is the ``func`` docstring:

.. autofunction:: texext.tests.for_docstrings.func
""")

    conf_source = """\
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.mathjax',
    'texext.math_dollar']
"""

    def test_docstrings(self):
        html = self.get_built_file('a_page.html')
        assert DOCSTRING_RE.search(html)



r""" A module to test docstring parsing with math such as $\gamma = \cos(\alpha)$

Need to test other markup - so: `a link <https://github.com>`_.
"""


def func():
    r""" A docstring with math in first line $z = \beta$

    With some more $a = 1$ math. Math across lines - $b
    = 2$.

    Further, there is a link_ and a |substitution|.

    .. _link: https://python.org
    .. |substitution| replace:: substituted
    """



""" Tests for plotcontext build using sphinx extensions

Test ability to combine plot_context with mathcode
"""

from os.path import join as pjoin

from .test_custom_plotdirective import TestCustomPlotDirective


class TestPlotContext(TestCustomPlotDirective):
    # Test build and output of custom_plotcontext project

    @classmethod
    def modify_pages(cls):
        conf_fname = pjoin(cls.page_source, 'conf.py')
        with open(conf_fname, 'rt') as fobj:
            contents = fobj.read()
        contents = contents.replace(
            "'matplotlib.sphinxext.plot_directive'",
            '"plot_directive"')
        contents += """
# Use mpl plot_context
mathcode_plot_context = 'matplotlib.sphinxext.plot_directive.plot_context'
"""



# emacs: -*- mode: python-mode; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
#
#   See LICENSE file distributed along with the texext package for the
#   copyright and license terms.
#
### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
""" Sphinx source processor to replace $a=b$ with :math:`a=b`
"""
from warnings import warn
import re
from functools import partial

from docutils import nodes
from docutils.utils import escape2null, unescape
from docutils.transforms import Transform

from sphinx.errors import ExtensionError
from sphinx.ext.mathbase import math


def d2m_source(source):
    r"""
    Replace dollar signs with :math:`` within strings in `source` list

    See: :func:`rst_dollars_to_math` for details.

    Parameters
    ----------
    source : sequence of str
        Sequence of strings, usually read from a ReST source file.  `source`
        modified in place.
    """
    s = "\n".join(source)
    source[:] = rst_dollars_to_math(s).split("\n")


class StringProtector(object):
    """ Replace / restore text in regexp groups with markers.

    Replace identified text with markers using ``obj.protect(in_str)``
    Restore replaced text in place of markers with ``obj.restore(in_str)``
    """

    def __init__(self, regexps):
        self.regexps = tuple(re.compile(r) for r in regexps)
        self._marker_contents = []
        # Marker unique to this instance
        self._marker_fmt = "___XXX_REPL_{:d}_{{:d}}___".format(id(self))

    def _repl(self, matchobj):
        content = matchobj.group(0)
        marker_contents = self._marker_contents
        marker = self._marker_fmt.format(len(marker_contents))
        marker_contents.append((marker, content))
        return marker

    def protect(self, in_str):
        out_str = in_str
        for regexp in self.regexps:
            out_str = regexp.sub(self._repl, out_str)
        return out_str

    def restore(self, in_str):
        # Change everything back that we pulled out
        # Put back in reverse order of removal.
        out_str = in_str
        for marker, content in self._marker_contents[::-1]:
            out_str = out_str.replace(marker, content)
        self._marker_contents = []
        return out_str

    def __add__(self, other):
        return self.__class__(self.regexps + other.regexps)


in_dollars_protector = StringProtector(
    (
    # This searches for "$blah$" inside a pair of curly braces --
    # don't change these, since they're probably coming from a nested
    # math environment.  So for each match, we replace it with a temporary
    # string, and later on we substitute the original back.
    re.compile(r"({[^{}$]*\$[^{}$]*\$[^{}]*})"),
    )
)

rst_protector = StringProtector(
    (
    # Line entirely containing backticks ending with optional whitespace
    # These happen in unusual heading underlines
    re.compile(r"^(`+\s*)$", flags=re.MULTILINE),
    # Anything between double backticks
    re.compile(r"(``[^`]*?``)"),
    # Anything between single backticks
    re.compile(r"(`[^`]*?`)"),
    )
) + in_dollars_protector


def rst_dollars_to_math(rst_str,
                        protector=rst_protector,
                        dollar_repl=r":math:`\1`"):
    """
    Replace dollar signs with backticks in string `rst_str`

    More precisely, do a regular expression search.  Replace a plain dollar
    sign ($) by a backtick (`).  Replace an escaped dollar sign (\$) by a
    dollar sign ($).  Don't change a dollar sign preceded or followed by a
    backtick (`$ or $`), because of strings like "``$HOME``".  Don't make any
    changes on lines starting with spaces, because those are indented and hence
    part of a block of code or examples.

    This also does not replace dollar signs enclosed in curly braces, to avoid
    nested math environments, such as::

      $f(n) = 0 \text{ if $n$ is prime}$

    Thus the above line would get changed to

      `f(n) = 0 \text{ if $n$ is prime}`

    Parameters
    ----------
    rst_str : str
        String, usually read from a ReST source file.
    protector : :class:`StringProtector` instance
        Object defining which regexps should be protected from dollar
        replacement.  Default is module global ``rst_protector`` instance.
    dollar_repl : string or callable
        Replacement expression for found text between (and including) dollars.

    Returns
    -------
    out_str : str
        Possibly modified string after replacing math dollar markers.
    """
    if rst_str.find("$") == -1:
        return rst_str
    out_str = protector.protect(rst_str)
    # matches $...$
    dollars = re.compile(r"(?<!\$)(?<!\\)\$([^\$]+?)\$")
    # regular expression for \$
    slashdollar = re.compile(r"\\\$")
    out_str = dollars.sub(dollar_repl, out_str)
    out_str = slashdollar.sub(r"$", out_str)
    return rst_protector.restore(out_str)


MATH_MARKER = "__D2M__"
txt_dollars_to_math = partial(rst_dollars_to_math,
                              protector=in_dollars_protector,
                              dollar_repl=MATH_MARKER + r"\1" + MATH_MARKER)


class MathDollarTransform(Transform):
    """ Replace text between dollars with inline math nodes
    """

    # Has to be before docutils.transforms.univerals.SmartQuotes transform,
    # which currently throws away the rawsource for the node.
    default_priority = 100

    def apply(self):
        for node in self.document.traverse(nodes.Text):
            self._process_node(node)

    def _process_node(self, node):
        # Avoid literals etc
        if not isinstance(node.parent, nodes.paragraph):
            return
        in_str = node.rawsource
        processed = txt_dollars_to_math(in_str)
        if MATH_MARKER not in processed:
            return
        parts = processed.split(MATH_MARKER)
        new_nodes = []
        for i, part in enumerate(parts):
            with_nulls = escape2null(part)
            to_backslashes = unescape(with_nulls, restore_backslashes=True)
            if part == '':
                continue
            if i % 2:  # See sphinx.ext.mathbase
                new_node = math(latex=to_backslashes)
            else:
                new_node = nodes.Text(unescape(with_nulls), to_backslashes)
            new_node.parent = node.parent
            new_nodes.append(new_node)
        # Put new nodes into parent's list of children
        new_children = []
        for child in node.parent.children:
            if not child is node:
                new_children.append(child)
            else:
                new_children += new_nodes
        node.parent.children = new_children


def mathdollar_docstrings(app, what, name, obj, options, lines):
    d2m_source(lines)


def setup(app):
    # Process pages after initial parse of ReST to avoid false positives
    app.add_transform(MathDollarTransform)
    try:
        # We have to process docstrings as text, it's the Wild Wild West.
        app.connect('autodoc-process-docstring', mathdollar_docstrings)
    except ExtensionError:
        warn("Need autodoc extension loaded for math_dollar to work on "
             "docstrings")



""" Texext package """

from . import math_dollar
from . import mathcode

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions


def setup(app):
    math_dollar.setup(app)
    mathcode.setup(app)



""" Sphinx extension to execute sympy code generating LaTeX

Uses code copied from sphinx/ext/mathbase.py.  That file has license:

    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
    :license: BSD, see LICENSE for details.

See README file for more information.

The MathCodeDirective accepts a Sympy expression as input.  It returns the
result of that expression as LaTeX included in a ``..math`` block.

This allows you do generate LaTeX dynamically.  For example:

    .. mathcode::

        import sympy
        a, b = sympy.symbols('a, b')
        a * 10 + 2 * b

The directive takes the result of ``sympy.latex(a * 10 + 2 * b)`` and puts it
inside a math directive in the ReST output.

By default the mathcode directives keep context from previous mathcode
directives, so you can use defined variables and functions.

You might want to clear the context for the first mathcode directive in a page,
by adding the ``:newcontext:`` option:

    .. mathcode::
        :newcontext:

        import sympy  # sympy no longer defined
        c, d = sympy.symbols('a, b')
        return c + d

If the last expression in the mathcode block is not an expression, the context
gets updated, but the extension generates no math directive to the output.
This allows you to have blocks that fill in calculations without rendering to
the page.  For example, this generates no output::

    .. mathcode::

        expr = a * 4

If you would like mathcode to share a namespace with the matplotlib ``plot``
directive, set the following in your ``conf.py``::

    # Config of mathcode directive
    mathcode_use_plot_ns = True

If you want to use the plot_directive context from within mathcode directives,
you need to list the plot_directive above the mathcode directive in your sphinx
extension list.  All the plot directives code will get run before all the
mathcode directive code.

Conversely, if you want to use the mathcode directive context from the
plot_directive, list mathcode first in your sphinx extension list.

Remember that, by default, the matplotlib ``plot`` directive will clear the
namespace context for each directive, so you may want to use the ``:context:``
option to the plot directive, most of the time.

If you want to work with a customized version of the plot_directive, you need
to supply the name of the plot context dictionary for the plot directive, as a
string.  For example, if you have a custom plot directive module importable as
``import my_path.plot_directive``, with the plot context in
``my_path.plot_directive.plot_context``, then your ``conf.py`` should have
lines like these::

    # Config of mathcode directive
    mathcode_plot_context = "my_path.plot_directive.plot_context"

The plot context is a string rather than the attribute itself in order to let
sphinx pickle the configuration between runs.  This allows sphinx to avoid
building pages that have not changed between calls to ``sphinx-build``.
"""

import warnings

from ast import parse, Expr, Expression

from docutils.parsers.rst import directives

try:
    from sphinx.ext.mathbase import MathDirective
except ImportError:  # Sphinx 1.8.0b1
    from sphinx.directives.patches import MathDirective


def eval_code(code_str, context):
    # Avoid depending on six unless running mathcode directive
    from six import exec_
    mod = parse(code_str, '<string>', 'exec')
    last_line = mod.body.pop() if isinstance(mod.body[-1], Expr) else None
    to_exec = compile(mod, '<string>', 'exec')
    exec_(to_exec, None, context)
    if last_line is None:
        return None
    to_eval = compile(Expression(last_line.value), '<string>', 'eval')
    return eval(to_eval, None, context)


class MathCodeDirective(MathDirective):
    """ Generate math environment from Sympy math expressions
    """
    optional_arguments = 0
    option_spec = {
        'label': directives.unchanged,
        'name': directives.unchanged,
        'nowrap': directives.flag,
        'newcontext': directives.flag,
    }

    def get_plot_directive(self):
        plot_directive = setup.config.mathcode_plot_directive
        if plot_directive is None:
            return None
        warnings.warn("mathcode_plot_directive deprecated; "
                      "please use mathcode_plot_context instead",
                      FutureWarning)
        return plot_directive

    def get_plot_context(self):
        # First try mathcode_plot_context dictionary
        plot_context = setup.config.mathcode_plot_context
        if plot_context is not None:
            # Plot context is a string naming a module attribute
            parts = plot_context.split('.')
            mod_name, el_name = '.'.join(parts[:-1]), parts[-1]
            mod = __import__(mod_name, globals(), locals(), el_name)
            return getattr(mod, el_name)
        # Next try getting dictionary from deprecated mathcode_plot_directive
        plot_directive = self.get_plot_directive()
        if plot_directive is not None:
            return plot_directive.plot_context
        # Default to matplotlib plot_context dictionary
        from matplotlib.sphinxext.plot_directive import plot_context
        return plot_context

    def get_context(self, newcontext=False):
        if setup.config.mathcode_use_plot_ns:
            plot_context = self.get_plot_context()
        else:
            plot_context = setup.code_context
        if newcontext:
            plot_context.clear()
        return plot_context

    def run(self):
        # Avoid depending on sympy unless running mathcode directive
        from sympy import latex
        want_new = True if 'newcontext' in self.options else False
        context = self.get_context(want_new)
        val = eval_code('\n'.join(self.content), context)
        if val is None:
            return []
        self.content = latex(val).splitlines()
        return super(MathCodeDirective, self).run()


def setup(app):
    # Global variables
    setup.app = app
    setup.config = app.config
    setup.confdir = app.confdir
    # Workspace for code run in Mathcode blocks
    setup.code_context = dict()
    app.add_directive('mathcode', MathCodeDirective)
    app.add_config_value('mathcode_use_plot_ns', False, 'env')
    app.add_config_value('mathcode_plot_context', None, 'env')
    # mathcode_plot_directive deprecated; prefer mathcode_plot_context
    app.add_config_value('mathcode_plot_directive', None, 'env')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
''' Installation script for texext package '''
import sys

# For some commands, use setuptools.
if len(set(('develop', 'bdist_egg', 'bdist_rpm', 'bdist', 'bdist_dumb',
            'install_egg_info', 'egg_info', 'easy_install', 'bdist_wheel',
            'bdist_mpkg')).intersection(sys.argv)) > 0:
    import setuptools

from distutils.core import setup

import versioneer

extra_setup_kwargs = ({} if 'setuptools' not in sys.modules else
                      dict(install_requires=['six', 'sphinx>=1.3.1']))


setup(name='texext',
      version=versioneer.get_version(),
      cmdclass=versioneer.get_cmdclass(),
      description='Sphinx extensions for working with LaTeX math',
      author='Ondřej Čertík, Matthew Brett',
      author_email='matthew.brett@gmail.com',
      maintainer='Matthew Brett, Ondřej Čertík',
      maintainer_email='matthew.brett@gmail.com',
      url='http://github.com/matthew-brett/texext',
      packages=['texext',
                'texext.tests'],
      package_data = {'texext': [
          'tests/tinypages/*.rst',
          'tests/tinypages/*.py',
          'tests/tinypages/_static/*',
          'tests/plotdirective/*.rst',
          'tests/plotdirective/*.py',
          'tests/plotdirective/_static/*',
          'tests/custom_plotcontext/*.rst',
          'tests/custom_plotcontext/*.py',
          'tests/custom_plotcontext/_static/*',
          'tests/custom_plotdirective/*.rst',
          'tests/custom_plotdirective/*.py',
          'tests/custom_plotdirective/_static/*']},
      license='BSD license',
      classifiers = [
            'Development Status :: 4 - Beta',
            'Environment :: Console',
            'Intended Audience :: Developers',
            'Intended Audience :: Science/Research',
            'License :: OSI Approved :: BSD License',
            'Programming Language :: Python',
            'Programming Language :: Python :: 3',
            'Topic :: Scientific/Engineering',
            'Operating System :: Microsoft :: Windows',
            'Operating System :: POSIX',
            'Operating System :: Unix',
            'Operating System :: MacOS',
        ],
      long_description = open('README.rst', 'rt').read(),
      **extra_setup_kwargs
      )



""" Tests for tinypages build using sphinx extensions """

from os.path import (join as pjoin, dirname, isdir)

import sphinx
SPHINX_ge_1p5 = sphinx.version_info[:2] >= (1, 5)

from sphinxtesters import PageBuilder

HERE = dirname(__file__)
PAGES = pjoin(HERE, 'tinypages')

from texext.tests.test_plotdirective import format_math_block

def _pdiff(str1, str2):
    # For debugging
    from difflib import ndiff
    print(''.join(ndiff(str1.splitlines(True), str2.splitlines(True))))


class TestTinyPages(PageBuilder):
    # Test build and output of tinypages project
    page_source_template = PAGES

    def test_some_math(self):
        assert isdir(self.out_dir)
        assert isdir(self.doctree_dir)
        doctree = self.get_doctree('some_math')
        assert len(doctree.document) == 1
        tree_str = self.doctree2str(doctree)
        if SPHINX_ge_1p5:
            back_ref = (
                '<paragraph>Refers to equation at '
                '<pending_xref refdoc="some_math" refdomain="math" '
                'refexplicit="False" reftarget="some-label" '
                'reftype="eq" refwarn="True">'
                '<literal classes="xref eq">some-label</literal>'
                '</pending_xref>')
        else:
            back_ref=(
                '<paragraph>Refers to equation at '
                '<eqref docname="some_math" '
                'target="some-label">(?)</eqref>')
        expected = (
            '<title>Some math</title>\n'
            '<paragraph>Here <math latex="a = 1"/>, except '
            '<title_reference>$b = 2$</title_reference>.</paragraph>\n'
            '<paragraph>Here <math latex="c = 3"/>, except '
            '<literal>$d = 4$</literal>.</paragraph>\n'
            '<literal_block xml:space="preserve">'
            'Here $e = 5$</literal_block>\n'
            '<bullet_list bullet="*">'
            '<list_item>'
            '<paragraph>'
            'A list item containing\n'
            '<math latex="f = 6"/> some mathematics.'
            '</paragraph>'
            '</list_item>'
            '<list_item>'
            '<paragraph>'
            'A list item containing '
            '<literal>a literal across\nlines</literal> '
            'and also <math latex="g = 7"/> some mathematics.'
            '</paragraph>'
            '</list_item>'
            '</bullet_list>\n'
            + format_math_block('some_math', "10 a + 2 b + q") +
            '\n<paragraph>More text</paragraph>\n'
            '<target refid="equation-some-label"/>\n'
            + format_math_block(
                'some_math', "5 a + 3 b",
                label='some-label',
                number='1',
                ids='equation-some-label') +
            '\n<paragraph>Yet more text</paragraph>\n'
            + format_math_block(
                "some_math", latex="5 w + 3 x") + '\n' +
            r'<paragraph>Math with <math latex="\beta"/> a backslash.'
            '</paragraph>\n'
            '<paragraph>'  # What happens to backslashes?
            'A protected whitespace with <math latex="dollars"/>.'
            '</paragraph>\n'
            '<paragraph>'
            'Some * asterisks *.  <math latex="dollars"/>. '
            'A line break.  Protected \ backslash.  '
            'Protected n in <math latex="a"/> line.</paragraph>\n'
            # Do labels get set as targets?
            + back_ref +
            '.</paragraph>')
        assert tree_str == expected


class TestTopLevel(TestTinyPages):
    # Test we can import math_dollar with just `texext`

    @classmethod
    def modify_source(cls):
        conf_fname = pjoin(cls.page_source, 'conf.py')
        with open(conf_fname, 'rt') as fobj:
            contents = fobj.read()
        contents = contents.replace("'texext.mathcode',\n", "")
        contents = contents.replace("'texext.math_dollar'", "'texext'")
        with open(conf_fname, 'wt') as fobj:
            fobj.write(contents)



""" Tests for plotdirective build using sphinx extensions

Test ability to combine plot_directive with mathcode
"""

from os.path import dirname, join as pjoin
import re

import sphinx
SPHINX_1p8 = sphinx.version_info[:2] >= (1, 8)

from sphinxtesters import PageBuilder

from texext.tests.test_plotdirective import EXP_PLOT_AND_MATH

PAGES = pjoin(dirname(__file__), 'plotdirective')


class TestCustomPlotDirective(PageBuilder):
    # Test build and output of custom_plotdirective project
    page_source_template = PAGES

    @classmethod
    def modify_pages(cls):
        conf_fname = pjoin(cls.page_source, 'conf.py')
        with open(conf_fname, 'rt') as fobj:
            contents = fobj.read()
        contents = contents.replace(
            "'matplotlib.sphinxext.plot_directive'",
            '"plot_directive"')
        contents += """
< # Use custom plot_directive
< sys.path.insert(0, abspath(pjoin('.')))
< import plot_directive
< mathcode_plot_directive = plot_directive
"""
        with open(conf_fname, 'wt') as fobj:
            fobj.write(contents)

    def test_plot_and_math(self):
        doctree = self.get_doctree('plot_and_math')
        assert len(doctree.document) == 1
        tree_str = self.doctree2str(doctree)
        # Sphinx by 1.3 adds "highlight_args={}", Sphinx at 1.1.3 does not
        assert re.compile(EXP_PLOT_AND_MATH).search(tree_str)
